from injecty import InjectyContext

from servey_stub.type_definition.dataclass_type_definition_factory import (
    DataclassTypeDefinitionFactory,
)
from servey_stub.type_definition.enum_type_definition_factory import (
    EnumTypeDefinitionFactory,
)
from servey_stub.type_definition.forward_ref_definition_factory import (
    ForwardRefDefinitionFactory,
)
from servey_stub.type_definition.generic_type_definition_factory import (
    GenericTypeDefinitionFactory,
)
from servey_stub.type_definition.primitive_type_definition_factory import (
    PrimitiveTypeDefinitionFactory,
)
from servey_stub.type_definition.type_definition_factory_abc import (
    TypeDefinitionFactoryABC,
)

priority = 100


def configure(context: InjectyContext):
    context.register_impls(
        TypeDefinitionFactoryABC,
        [
            DataclassTypeDefinitionFactory,
            PrimitiveTypeDefinitionFactory,
            EnumTypeDefinitionFactory,
            GenericTypeDefinitionFactory,
            ForwardRefDefinitionFactory,
        ],
    )



from typing import Type, Optional, ForwardRef

from marshy.utils import resolve_forward_refs

from servey_stub.type_definition.imports_definition import ImportsDefinition
from servey_stub.type_definition.type_definition import TypeDefinition
from servey_stub.type_definition.type_definition_context import TypeDefinitionContext
from servey_stub.type_definition.type_definition_factory_abc import (
    TypeDefinitionFactoryABC,
)


class ForwardRefDefinitionFactory(TypeDefinitionFactoryABC):
    def create_type_definition(
        self, type_: Type, context: TypeDefinitionContext
    ) -> Optional[TypeDefinition]:
        if getattr(type_, "__class__", None) != ForwardRef:
            return

        resolved = resolve_forward_refs(type_)
        type_definition = context.get_type_definition(resolved)

        imports = ImportsDefinition()
        imports.add("typing.ForwardRef")
        ref = ".".join(type_definition.imports.imports[0])
        return TypeDefinition(f'ForwardRef("{ref}")', imports)
